/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.ldap;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;

public class LdapSupportFileSys implements LdapSupportService {

    File root;

    public LdapSupportFileSys(File root) {
        this.root = root;
    }

    @Override
    public <T extends HasCommonLdapFields> List<T> findChildNodes(T org) {
        List<T> result = new ArrayList<T>();

        List<String> path = Arrays.asList(org.getDn().split(Pattern.quote(",")));
        Collections.reverse(path);

        File finding = findDirectory(root, path);

        for (File child : finding.listFiles()) {
            if (child.isDirectory()) {
                try {
                    T item = (T) org.getClass().newInstance();
                    List<String> localPath = new ArrayList<String>(path);
                    localPath.add(0, child.getName());
                    item.setDn(merge(localPath, ","));

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        return result;
    }

    File findDirectory(File start, List<String> steps) {
        String path = toFilePath(steps);
        File result = new File(root.getAbsolutePath() + File.separator + toFilePath(steps));
        return result;
    }

    String toFilePath(List<String> parts) {
        return merge(parts, File.separator);
    }

    String merge(List<String> parts, String junctor) {
        StringBuilder sb = new StringBuilder();
        for (String part : parts) {
            sb.append(part);
            sb.append(junctor);
        }
        if (parts.size() > 1) {
            sb.delete(sb.length() - junctor.length(), sb.length());
        }
        return sb.toString();
    }

    @Override
    public <T extends HasCommonLdapFields> List<T> find(T org) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public <T extends HasCommonLdapFields, R> List<R> find(T org, Class<R> resultType) {
        // TODO Auto-generated method stub
        return null;
    }

}
